import numpy as np
import matplotlib.pyplot as plt
from signal_processing.hht import hsa
from scipy.signal import stft,cwt,ricker
from tftb.generators import amgauss
from time import time

def gen_hs(batch_size,n_batches):
    fs = 100
    t_begin = 0.0
    t_end = 2.0
    N = int(t_end-t_begin)*fs
    f_begin = 1.0
    f_end = 40.0
    df = 0.5

    f1 = 5
    f2 = 17
    f3 = 29
    
    widths = np.arange(f_begin,f_end,df)

    for i in range(n_batches):
        print('Batch {}/{}'.format(i+1,n_batches))

        x_stft = np.zeros((batch_size,11,21))
        x_cwt = np.zeros((batch_size,len(widths),N))
        x_hht = np.zeros((batch_size,len(widths),N))
        y = np.zeros((batch_size,3),dtype=int)

        CMA = 0 # Moving average of processing time
        for j in range(batch_size):
            print('Sample {}/{} ({:.2f} s/step)'.format(j+1,batch_size,CMA))
            start_time = time()
            t = np.linspace(t_begin,t_end,N)
            s1 = np.sin(2*np.pi * (f1+np.random.normal(0,2))*t)
            s2 = np.sin(2*np.pi * (f2+np.random.normal(0,2))*t)
            s3 = np.sin(2*np.pi * (f3+np.random.normal(0,2))*t)
            s_list = [s1,s2,s3]

            class_idx = np.random.randint(3)
            y[j,class_idx] = 1
            s_list[class_idx] *= 1 + amgauss(N,np.random.normal(N/2,2*np.sqrt(N)))

            s = sum(s_list) + np.random.normal(0.0,0.5,N)
            hs = hsa.hht(s,fs,widths)
            scalegram = cwt(s,ricker,widths)
            _,_,specgram = stft(s,fs=fs,nperseg=N/10,return_onesided=True)

            x_stft[j] = np.abs(specgram)
            x_cwt[j] = scalegram
            x_hht[j] = hs
            dt = time() - start_time
            CMA = (dt + j*CMA)/(j+1)
        np.save('data/synthetic/hht/x_{:03d}.npy'.format(i),x_hht)
        np.save('data/synthetic/cwt/x_{:03d}.npy'.format(i),x_cwt)
        np.save('data/synthetic/stft/x_{:03d}.npy'.format(i),x_stft)
        np.save('data/synthetic/y_{:03d}.npy'.format(i),y)


if __name__ == '__main__':
    hs = gen_hs(1000,10)
