import numpy as np
import matplotlib.pyplot as plt
from scipy.signal import stft, chirp, cwt, ricker, hilbert
from signal_processing.hht import emd, visualization

fs = 200
t0 = 0
t1 = 2
N = int((t1-t0)*fs)

t = np.linspace(t0,t1,N)
s = np.sin(2*np.pi*40*t) + chirp(t,1,t[-1],20)

plt.clf()
plt.plot(t,s)
plt.xlabel('Time [s]')
plt.savefig('nonstationary.png')

_f,_t,Zxx = stft(s,fs,nperseg=64)

plt.clf()
plt.pcolor(_t,_f,np.abs(Zxx))
plt.xlabel('Time [s]')
plt.ylabel('Frequency [Hz]')
plt.savefig('spectrogram.png')

widths = np.arange(0.1,50,0.1)
cwtmatr = cwt(s,ricker,widths)

plt.clf()
plt.pcolor(t,widths,cwtmatr)
plt.xlabel('Time [s]')
plt.ylabel('Scale')
plt.savefig('cwt.png')

hs = hilbert(s)
inst_phase = np.unwrap(np.angle(hs)) * fs/(2*np.pi)
inst_freq = np.diff(inst_phase)

plt.clf()
plt.plot(t[1:],inst_freq)
plt.xlabel('Time [s]')
plt.ylabel('Instantaneous Frequency [Hz]')
plt.savefig('nonsense.png')

imfs = emd.decompose(s)
visualization.plot_emd(s,imfs)
