import numpy as np
import matplotlib.pyplot as plt
from signal_processing.emd import hsa
from time import time

def process(subject_name,sensors):
    print('Preprocessing data for subject \'{}\'...'.format(subject_name))
    fs = 100 # Hz
    num_channels = 118
    widths = np.arange(0,50,0.5)

    print('{} Storing data as ndarray...'.format(subject_name))
    start = time()

    num_samples = 0
    with open('data/ascii/data_set_IVa_{}_cnt.txt'.format(subject_name)) as f:
        for _ in f:
            num_samples += 1
    t = np.linspace(0,num_samples/fs,num_samples)

    # Store raw data as ndarray
    raw_data = np.zeros((118,num_samples))
    data_file = open('data/ascii/data_set_IVa_{}_cnt.txt'.format(subject_name))
    for i,line in enumerate(data_file):
        raw_data[:,i] = np.array([0.1*float(x) for x in line.split()])
    print('{} Stored data in {:.2f} seconds'.format(subject_name,time() - start))

    print('{} Performing Hilbert-Huang transform...'.format(subject_name))
    num_sequences = 0
    with open('data/ascii/data_set_IVa_{}_mrk.txt'.format(subject_name)) as f:
        for line in f:
            if line.split()[1] != 'NaN':
                num_sequences += 1
    label_file = open('data/ascii/data_set_IVa_{}_mrk.txt'.format(subject_name))
    label_array = np.zeros((num_sequences,2), dtype=int)
    feature_array = np.zeros((num_sequences,len(widths),350,len(sensors)))
    print((num_sequences,len(widths),350,len(sensors)))
    for i,line in enumerate(label_file):
        line = line.split()
        if line[1] == 'NaN':
            # Does not contain label data and should be ignored
            continue
        print('{} Processing sequence {}/{}'.format(subject_name,i+1,num_sequences))
        start_seq = time()
        if line[1] == '1':
            label_array[i,0] = 1
        elif line[1] == '2':
            label_array[i,1] = 1
        t0_idx = int(line[0]) # Start of cue
        t1_idx = int(t0_idx + 3.5*fs) # Each cue remains for 3.5s
        data = raw_data[:,t0_idx:t1_idx]
        old_time_chan = time()
        for j,sensor in enumerate(sensors):
            time_chan = time()
            print('{} {}/{} Performing HHT for channel {}... ({:.2f} s/channel)'.format(subject_name,i+1,num_sequences,sensor,time_chan - old_time_chan))
            hs = hsa.hht(data[sensor],fs,widths)
            feature_array[i,:,:,j] = hs
            old_time_chan = time_chan
        print('{} Processed sequence {}/{} in {:.2f} seconds'.format(subject_name,i+1,num_sequences,time() - start_seq))
    np.save('data/numpy/{}_features.npy'.format(subject_name),feature_array)
    np.save('data/numpy/{}_labels.npy'.format(subject_name),label_array)

start = time()
subjects = ['aa','al','av','aw','ay']
# Use sensors at Fp1, Fp2, F3, F4, C3, C4, O1 and O2
sensors = [0,4,15,19,51,55,111,112]
for name in subjects:
    process(name,sensors)
print('Finished!')
print('Total time: {:.2f} seconds'.format(time() - start))
