import numpy as np
from scipy.signal import hilbert
from signal_processing.hht import emd

def hht(signal,fs,widths=None):
    if widths is None:
        widths = np.linspace(0.0,fs/2,signal.size)

    imfs = emd.decompose(signal)
    hilbert_spectrum = np.zeros((widths.size,signal.size))
    for imf in imfs[1:-1]:
        imf_analytic = hilbert(imf)
        inst_ampl = np.abs(imf_analytic)
        inst_phase = np.unwrap(np.angle(imf_analytic)) * fs/(2.0*np.pi)
        inst_freq = np.diff(inst_phase)
        for i,freq in enumerate(inst_freq):
            if(i == 0):
                continue # Skip first value
            for j in range(widths.size):
                if(widths[j-1] < freq < widths[j]):
                    hilbert_spectrum[j,i] += inst_ampl[i]
    return hilbert_spectrum
