import numpy as np
from scipy.signal import hilbert, find_peaks_cwt


def get_mask(imf,fs):
    cutoff = 0.1 # Cutting off at the edges to avoid boundary effects

    hs = hilbert(imf)
    inst_ampl = np.abs(hs)
    omega = np.unwrap(np.angle(hs))
    inst_freq = fs/(2*np.pi) * np.diff(omega)

    cutoff_left = int(len(inst_ampl)*cutoff)
    cutoff_right = int(len(inst_ampl)*(1 - cutoff))

    inst_ampl = inst_ampl[cutoff_left:cutoff_right]
    inst_freq = inst_freq[cutoff_left:cutoff_right]

    K_max = np.max(inst_ampl)
    K_min = np.min(inst_ampl)
    F_max = np.max(inst_freq)
    F_min = np.min(inst_freq)

    # delta_f is equivalent to peaks per second (pps)
    # using low pass filtering so that every local maxima may be considered a peak
    # TODO: More robust peak detection algorithm
    ##inst_freq_lp = 0.5*(inst_freq[:-1] + inst_freq[1:])
    ##inst_freq_lp = 0.5*(inst_freq_lp[:-1] + inst_freq_lp[1:])
    n = 0
    pps_list = []
    peak_width = np.arange(fs/100,fs/10)
    while n + fs < len(inst_freq):
        interval = inst_freq[n:n+fs]
        pps_list.append(np.size(find_peaks_cwt(interval,peak_width)))
        n = n + fs
    delta_f = int(np.median(pps_list))
    print(delta_f)

    # Estimate amplitude and frequency of the two modes
    a1 = 0.5*(K_max + K_min)
    a2 = K_max - a1
    f2 = F_min - a1*delta_f/(a1 + a2)
    f1 = delta_f + f2

    print('f1: {0:.2f} f2: {1:.2f}'.format(f1,f2))
    print('a1: {0:.2f} a2: {1:.2f}'.format(a1,a2))

    # To seperate the two modes, a masking frequency satisfying
    # f1/fm > 0.7 and f2/fm < 0.6 is appropriate
    fm = 0.5*(f2/0.6 + f1/0.7)

    print('f1/fm = {0:.2f}'.format(f1/fm))
    print('f2/fm = {0:.2f}'.format(f2/fm))

    return 2.5, fm
