import numpy as np
from scipy.signal import argrelmax, argrelmin
from scipy.interpolate import CubicSpline

import matplotlib.pyplot as plt

def count_extrema(x):
    return np.size(argrelmax(x)) + np.size(argrelmin(x))

def count_zero_crossings(x):
    return (np.diff(np.sign(x)) != 0).sum()

def normalize_imfs(imfs, n_scalings=4, threshold=0.001):
    num_imfs = len(imfs)

    scaled_imfs = np.ndarray(imfs.shape, np.float)
    inst_amp = np.ndarray(imfs.shape, np.float)

    for i in range(num_imfs):
        scaled_imfs[i], inst_amp[i] = scale_imf(imfs[i], n_scalings, threshold)

    return scaled_imfs, inst_amp

def scale_imf(imf, n_scalings=4, threshold=0.001):
    n = len(imf)
    correction_curves = np.ndarray((n_scalings,n), np.float)
    scaled_imf = imf

    for i in range(n_scalings):
        abs_val = np.fabs(scaled_imf)
        x = argrelmax(abs_val)[0]
        y = abs_val[x]
        correction_curve = CubicSpline(x,y)(np.arange(n))

        for j in range(n):
            if correction_curve[j] == threshold:
                if j > 0:
                    correction_curve[j] = correction_curve[j-1]
                else:
                    correction_curve[j] = scaled_imf[j]
            scaled_imf[j] = scaled_imf[j] / correction_curve[j]

        correction_curves[i] = correction_curve

    inst_amp = correction_curves[0]
    if n_scalings > 1:
        for i in range(1,len(correction_curves)):
            inst_amp = np.multiply(inst_amp, correction_curves[i])

    return scaled_imf, inst_amp
